@@ -14,7 +14,7 @@ macro_rules! ruby_version_v_1_9_1(
             use std;
             use anyhow::{Context, format_err, Result};
             use bindings::$ruby_version::*;
-            use crate::core::types::ProcessMemory;
+            use crate::core::process::ProcessMemory;
 
             get_stack_trace!(rb_thread_struct);
             get_execution_context_from_thread!(rb_thread_struct);
@@ -37,7 +37,7 @@ macro_rules! ruby_version_v_1_9_2_to_3(
             use std;
             use anyhow::{Context, format_err, Result};
             use bindings::$ruby_version::*;
-            use crate::core::types::ProcessMemory;
+            use crate::core::process::ProcessMemory;
 
             get_stack_trace!(rb_thread_struct);
             get_execution_context_from_thread!(rb_thread_struct);
@@ -59,7 +59,7 @@ macro_rules! ruby_version_v_2_0_to_2_2(
            use std;
            use anyhow::{Context, format_err, Result};
            use bindings::$ruby_version::*;
-           use crate::core::types::ProcessMemory;
+           use crate::core::process::ProcessMemory;
 
             // These 4 functions are the
             // core of how the program works. They're essentially a straight port of
@@ -93,7 +93,7 @@ macro_rules! ruby_version_v_2_3_to_2_4(
            use std;
            use anyhow::{Context, format_err, Result};
            use bindings::$ruby_version::*;
-           use crate::core::types::ProcessMemory;
+           use crate::core::process::ProcessMemory;
 
             get_stack_trace!(rb_thread_struct);
             get_execution_context_from_thread!(rb_thread_struct);
@@ -115,7 +115,7 @@ macro_rules! ruby_version_v2_5_x(
            use std;
            use anyhow::{Context, format_err, Result};
            use bindings::$ruby_version::*;
-           use crate::core::types::ProcessMemory;
+           use crate::core::process::ProcessMemory;
 
             get_stack_trace!(rb_execution_context_struct);
             get_execution_context_from_thread!(rb_execution_context_struct);
@@ -141,7 +141,7 @@ macro_rules! ruby_version_v2_6_x(
            use std;
            use anyhow::{Context, format_err, Result};
            use bindings::$ruby_version::*;
-           use crate::core::types::ProcessMemory;
+           use crate::core::process::ProcessMemory;
 
             get_stack_trace!(rb_execution_context_struct);
             get_execution_context_from_thread!(rb_execution_context_struct);
@@ -167,7 +167,7 @@ macro_rules! ruby_version_v2_7_x(
             use std;
             use anyhow::{Context, format_err, Result};
             use bindings::$ruby_version::*;
-            use crate::core::types::ProcessMemory;
+            use crate::core::process::ProcessMemory;
 
             get_stack_trace!(rb_execution_context_struct);
             get_execution_context_from_thread!(rb_execution_context_struct);
@@ -190,7 +190,7 @@ macro_rules! ruby_version_v3_0_x(
             use std;
             use anyhow::{Context, format_err, Result};
             use bindings::$ruby_version::*;
-            use crate::core::types::ProcessMemory;
+            use crate::core::process::ProcessMemory;
 
             get_stack_trace!(rb_execution_context_struct);
             get_execution_context_from_vm!();
@@ -268,8 +268,8 @@ macro_rules! get_execution_context_from_vm(
 
 macro_rules! get_stack_trace(
     ($thread_type:ident) => (
-        use crate::core::types::*;
-        use crate::core::types::StackFrame;
+        use crate::core::process::Pid;
+        use crate::core::types::{StackFrame, StackTrace, MemoryCopyError};
 
         pub fn get_stack_trace<T: ProcessMemory>(
             ruby_current_thread_address_location: usize,
@@ -809,7 +809,7 @@ macro_rules! get_cfunc_name(
                 imemo_type_imemo_ment => Ok(&imemo as *const rb_method_entry_struct),
                 imemo_type_imemo_svar => {
                     let svar: vm_svar = source.copy_struct(raw_imemo).context(raw_imemo)?;
-                    check_method_entry(svar.cref_or_me, source)
+                    check_method_entry(svar.cref_or_me as usize, source)
                 },
                 _ => Ok(raw_imemo as *const rb_method_entry_struct)
             }
@@ -994,15 +994,19 @@ ruby_version_v2_6_x!(ruby_2_6_5);
 ruby_version_v2_6_x!(ruby_2_6_6);
 ruby_version_v2_6_x!(ruby_2_6_7);
 ruby_version_v2_6_x!(ruby_2_6_8);
+ruby_version_v2_6_x!(ruby_2_6_9);
 ruby_version_v2_7_x!(ruby_2_7_0);
 ruby_version_v2_7_x!(ruby_2_7_1);
 ruby_version_v2_7_x!(ruby_2_7_2);
 ruby_version_v2_7_x!(ruby_2_7_3);
 ruby_version_v2_7_x!(ruby_2_7_4);
+ruby_version_v2_7_x!(ruby_2_7_5);
 ruby_version_v3_0_x!(ruby_3_0_0);
 ruby_version_v3_0_x!(ruby_3_0_1);
 ruby_version_v3_0_x!(ruby_3_0_2);
+ruby_version_v3_0_x!(ruby_3_0_3);
 
+#[cfg(not(debug_assertions))]
 #[cfg(test)]
 mod tests {
     use rbspy_testdata::*;
@@ -1313,6 +1317,22 @@ mod tests {
         assert_eq!(real_stack_trace_2_7_2(), stack_trace.trace);
     }
 
+    #[cfg(target_pointer_width = "64")]
+    #[test]
+    fn test_get_ruby_stack_trace_2_7_5() {
+        let current_thread_addr = 0x7fdd8d626070;
+        let global_symbols_addr = Some(0x7fdd8d60eb80);
+        let stack_trace = ruby_version::ruby_2_7_5::get_stack_trace::<CoreDump>(
+            current_thread_addr,
+            0,
+            global_symbols_addr,
+            &coredump_2_7_2(),
+            0,
+        )
+        .unwrap();
+        assert_eq!(real_stack_trace_2_7_2(), stack_trace.trace);
+    }
+
     #[cfg(target_pointer_width = "64")]
     #[test]
     fn test_get_ruby_stack_trace_3_0_0() {
@@ -1363,4 +1383,21 @@ mod tests {
         .unwrap();
         assert_eq!(real_stack_trace_2_7_2(), stack_trace.trace);
     }
+
+    #[cfg(target_pointer_width = "64")]
+    #[test]
+    fn test_get_ruby_stack_trace_3_0_3() {
+        let source = coredump_3_0_0();
+        let vm_addr = 0x7fdacdab7470;
+        let global_symbols_addr = Some(0x7fdacdaa9d80);
+        let stack_trace = ruby_version::ruby_3_0_3::get_stack_trace::<CoreDump>(
+            0,
+            vm_addr,
+            global_symbols_addr,
+            &source,
+            0,
+        )
+        .unwrap();
+        assert_eq!(real_stack_trace_2_7_2(), stack_trace.trace);
+    }
 }
